/* -*- Mode: c++; c-basic-offset: 4; tab-width: 40; indent-tabs-mode: nil -*- */
/* vim: set ts=40 sw=4 et tw=99: */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Mozilla SpiderMonkey property tree implementation
 *
 * The Initial Developer of the Original Code is
 *   Mozilla Foundation
 * Portions created by the Initial Developer are Copyright (C) 2002-2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Brendan Eich <brendan@mozilla.org>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef jspropertytree_h___
#define jspropertytree_h___

#include "jsarena.h"
#include "jsdhash.h"
#include "jsprvtd.h"

struct JSScope;

namespace js {

JSDHashOperator RemoveNodeIfDead(JSDHashTable *table, JSDHashEntryHdr *hdr,
                                 uint32 number, void *arg);

void SweepScopeProperties(JSContext *cx);

class PropertyTree
{
    friend struct ::JSScope;
    friend void js::SweepScopeProperties(JSContext *cx);

    JSDHashTable        hash;
    JSScopeProperty     *freeList;
    JSArenaPool         arenaPool;
    uint32              emptyShapeChanges;

    bool insertChild(JSContext *cx, JSScopeProperty *parent, JSScopeProperty *child);
    void removeChild(JSContext *cx, JSScopeProperty *child);
    void emptyShapeChange(uint32 oldEmptyShape, uint32 newEmptyShape);

  public:
    bool init();
    void finish();

    JSScopeProperty *newScopeProperty(JSContext *cx, bool gcLocked = false);

    JSScopeProperty *getChild(JSContext *cx, JSScopeProperty *parent, uint32 shape,
                              const JSScopeProperty &child);
};

} /* namespace js */

#endif /* jspropertytree_h___ */
